from ai21.modules.resources.nlp_task import NLPTask
from ai21.utils import validate_mandatory_field


class Tokenization(NLPTask):

    MODULE_NAME = 'tokenize'

    @classmethod
    def execute(cls, **params):
        validate_mandatory_field(key='text', call_name=cls.MODULE_NAME, params=params, validate_type=True, expected_type=str)
        url = f'{cls.get_base_url(**params)}/{cls.MODULE_NAME}'
        return super().execute(task_url=url, **params)








from ai21.ai21_studio_client import AI21StudioClient
from ai21.modules.resources.ai21_module import AI21Module


class ListableResource(AI21Module):

    @classmethod
    def list(cls, **params):
        url = cls.get_module_url(**params)
        client = AI21StudioClient(**params)
        return client.execute_http_request(method='GET', url=url)






from ai21.ai21_studio_client import AI21StudioClient
from ai21.modules.resources.ai21_module import AI21Module


class RetrievableResource(AI21Module):

    @classmethod
    def get(cls, object_id: str, **params):
        url = f'{cls.get_module_url(**params)}/{object_id}'
        client = AI21StudioClient(**params)
        return client.execute_http_request(method='GET', url=url)






from typing import Dict
import logging
import ai21
from ai21.ai21_object import construct_ai21_object
from ai21.errors import MissingInputException, WrongInputTypeException, UnsupportedInputException

logger = logging.getLogger("ai21")


def log_info(message):
    if ai21.log_level == "debug":
        print(message)
        logger.debug(message)
    elif ai21.log_level == "info":
        logger.info(message)


def log_error(message):
    logger.error(message)


def convert_to_ai21_object(obj: Dict):
    if isinstance(obj, list):
        return [construct_ai21_object(i) for i in obj]

    return construct_ai21_object(obj)


def validate_mandatory_field(key: str, call_name: str, params: Dict, validate_type: bool = False, expected_type: type = None):
    value = params.get(key, None)
    if value is None:
        raise MissingInputException(field_name=key, call_name=call_name)
    if not validate_type:
        pass
    if not isinstance(value, expected_type):
        raise WrongInputTypeException(key=key, expected_type=expected_type, given_type=type(value))


def validate_mandatory_fields(key_to_type: Dict, params: Dict, call_name: str):
    [validate_mandatory_field(key=key, call_name=call_name, params=params, validate_type=True, expected_type=key_type)
     for key, key_type in key_to_type]


def validate_unsupported_field(key: str, call_name: str, params: Dict):
    if key in params:
        raise UnsupportedInputException(field_name=key, call_name=call_name)


def get_global_configs():
    from ai21 import api_key, organization, application, api_version, api_host, timeout_sec, num_retries
    return {
        'api_key': api_key,
        'organization': organization,
        'application': application,
        'api_version': api_version,
        'api_host': api_host,
        'timeout_sec': timeout_sec,
        'num_retries': num_retries
    }




def supported_resources():
    from ai21 import Completion, Dataset, CustomModel, Paraphrase, Tokenization, Summarize, Segmentation, Improvements, GEC

    return {
        'completion': Completion,
        'tokenization': Tokenization,
        'dataset': Dataset,
        'customModel': CustomModel,
        'paraphrase': Paraphrase,
        'summarization': Summarize,
        "segmentation": Segmentation,
        "improvements": Improvements,
        "gec": GEC
    }



from ai21.constants import SAGEMAKER_ENDPOINT_KEY
from ai21.modules.resources.nlp_task import NLPTask
from ai21.utils import validate_mandatory_field


class Completion(NLPTask):
    MODULE_NAME = 'complete'

    @classmethod
    def execute(cls, experimental_mode=False, **params):
        validate_mandatory_field(key='prompt', call_name=cls.MODULE_NAME, params=params, validate_type=True, expected_type=str)
        if params.get('stopSequences', None) is None:
            params['stopSequences'] = []

        use_sm_endpoint = SAGEMAKER_ENDPOINT_KEY in params
        if use_sm_endpoint:
            return cls.execute_sm_endpoint(**params, unsupported_fields=['model', 'custom_model'])
        return cls.execute_studio_api(experimental_mode, **params)

    @classmethod
    def execute_studio_api(cls, experimental_mode: False, **params):
        validate_mandatory_field(key='model', call_name=cls.MODULE_NAME, params=params, validate_type=True, expected_type=str)
        model = params.get('model', None)
        custom_model = params.get('custom_model', None)
        if experimental_mode:
            model = f'experimental/{model}'
        url = f'{cls.get_base_url(**params)}/{model}'
        if custom_model is not None:
            url = f'{url}/{custom_model}'
        url = f'{url}/{cls.MODULE_NAME}'
        return super().execute(task_url=url, **params)



from ai21 import Completion
from ai21.modules.resources.nlp_task import NLPTask
from ai21.utils import validate_mandatory_field


class Experimental(NLPTask):

    @classmethod
    def _execute(cls, module: str, **params):
        url = f'{cls.get_base_url(**params)}/experimental/{module}'
        return super().execute(task_url=url, **params)

    @classmethod
    def rewrite(cls, **params):
        validate_mandatory_field(key='text', call_name="rewrite_experimental", params=params, validate_type=True, expected_type=str)
        url = cls.get_base_url(**params)
        url = f'{url}/experimental/rewrite'
        return super().execute(task_url=url, **params)

    @classmethod
    def summarize(cls, **params):
        validate_mandatory_field(key='text', call_name="summarize_experimental", params=params, validate_type=True, expected_type=str)
        url = cls.get_base_url(**params)
        url = f'{url}/experimental/summarize'
        return super().execute(task_url=url, **params)

    @classmethod
    def j1_grande_instruct(cls, **params):
        params["model"] = "j1-grande-instruct"
        return Completion.execute(experimental_mode=True, **params)



from ai21.constants import STUDIO_HOST, DEFAULT_API_VERSION
from ai21.modules.completion import Completion
from ai21.modules.dataset import Dataset
from ai21.modules.tokenization import Tokenization
from ai21.modules.custom_model import CustomModel
from ai21.modules.experimantal import Experimental
from ai21.modules.paraphrase import Paraphrase
from ai21.modules.summarize import Summarize
from ai21.modules.segmentation import Segmentation
from ai21.modules.improvements import Improvements
from ai21.modules.gec import GEC


api_key = None
organization = None
application = None
api_version = DEFAULT_API_VERSION
api_host = STUDIO_HOST
timeout_sec = None
num_retries = None
log_level = 'error'





import json
import re

import boto3
from botocore.exceptions import ClientError

from ai21.errors import ServerError, ServiceUnavailable, BadRequest, APIError
from ai21.http_client import handle_non_success_response
from ai21.utils import convert_to_ai21_object, log_error

sm_runtime = boto3.client("sagemaker-runtime")


def invoke_sm_endpoint(endpoint_name: str, input_json: str):
    try:
        response = sm_runtime.invoke_endpoint(
            EndpointName=endpoint_name,
            ContentType="application/json",
            Accept="application/json",
            Body=input_json,
        )
    except ClientError as sm_client_error:
        handle_sagemaker_client_error(sm_client_error)
    except Exception as exception:
        log_error(f'Calling {endpoint_name} failed with Exception: {exception}')
        raise exception
    response_body = json.load(response["Body"])
    return convert_to_ai21_object(response_body)


def handle_sagemaker_client_error(client_exception: ClientError):
    error_response = client_exception.response
    error_message = error_response.get('Error', {}).get('Message', '')
    status_code = error_response.get('ResponseMetadata', {}).get('HTTPStatusCode', None)
    # According to https://docs.aws.amazon.com/sagemaker/latest/APIReference/API_runtime_InvokeEndpoint.html#API_runtime_InvokeEndpoint_Errors
    if status_code == 400:
        raise BadRequest(details=error_message)
    if status_code == 424:
        error_message_template = re.compile(r'Received client error \((.*?)\) from primary with message \"(.*?)\". See .* in account .* for more information.')
        model_status_code = int(error_message_template.search(error_message).group(1))
        model_error_message = error_message_template.search(error_message).group(2)
        handle_non_success_response(model_status_code, model_error_message)
    if status_code == 429 or status_code == 503:
        raise ServiceUnavailable(details=error_message)
    if status_code == 500:
        raise ServerError(details=error_message)
    raise APIError(status_code, details=error_message)



from typing import List

from ai21.errors import EmptyMandatoryListException
from ai21.modules.resources.nlp_task import NLPTask
from ai21.utils import validate_mandatory_field


class Improvements(NLPTask):
    MODULE_NAME = 'improvements'

    @classmethod
    def execute(cls, **params):
        validate_mandatory_field(key='text', call_name=cls.MODULE_NAME, params=params, validate_type=True, expected_type=str)
        if params.get('types') is None or len(params['types']) == 0:
            raise EmptyMandatoryListException('types')
        url = cls.get_base_url(**params)
        url = f'{url}/{cls.MODULE_NAME}'
        return super().execute(task_url=url, **params)



from ai21.ai21_studio_client import AI21StudioClient
from ai21.modules.resources.ai21_module import AI21Module


class UploadResource(AI21Module):

    @classmethod
    def upload(cls, file_path: str, file_param_name: str, **params):
        files = {file_param_name: open(file_path, 'rb')}
        url = cls.get_module_url(**params)
        client = AI21StudioClient(**params)
        return client.execute_http_request(method='POST', url=url, params=params, files=files)






from typing import Optional


class APIError(Exception):
    def __init__(self, status_code: int, details: Optional[str] = None):
        super().__init__(details)
        self.details = details
        self.status_code = status_code

    def __str__(self) -> str:
        return f'Failed with http status code: {self.status_code} ({type(self).__name__}). Details: {self.details}'


class BadRequest(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(400, details)


class Unauthorized(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(401, details)


class UnprocessableEntity(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(422, details)


class TooManyRequests(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(429, details)


class ServerError(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(500, details)


class ServiceUnavailable(APIError):
    def __init__(self, details: Optional[str] = None):
        super().__init__(500, details)


class AI21ClientException(Exception):
    def __init__(self, message: str):
        self.message = message
        super().__init__(message)

    def __str__(self) -> str:
        return f'{type(self).__name__} {self.message}'


class MissingInputException(AI21ClientException):
    def __init__(self, field_name: str, call_name: str):
        message = f'{field_name} is required for the {call_name} call'
        super().__init__(message)


class UnsupportedInputException(AI21ClientException):
    def __init__(self, field_name: str, call_name: str):
        message = f'{field_name} is unsupported for the {call_name} call'
        super().__init__(message)


class OnlyOneInputException(AI21ClientException):
    def __init__(self, field_name1: str, field_name2: str, call_name: str):
        message = f'{field_name1} or {field_name2} is required for the {call_name} call, but not both'
        super().__init__(message)


class WrongInputTypeException(AI21ClientException):
    def __init__(self, key: str, expected_type: type, given_type: type):
        message = f'Supplied {key} should be {expected_type}, but {given_type} was passed instead'
        super().__init__(message)


class EmptyMandatoryListException(AI21ClientException):
    def __init__(self, key: str):
        message = f'Supplied {key} is empty. At least one element should be present in the list'
        super().__init__(message)


class MissingApiKeyException(AI21ClientException):
    def __init__(self):
        message = 'API key must be supplied either globally in the ai21 namespace, or to be provided in the call args'
        super().__init__(message)
        self.message = message



import json

from ai21.ai21_studio_client import AI21StudioClient
from ai21.modules.resources.ai21_module import AI21Module
from ai21.utils import validate_mandatory_field, validate_unsupported_field


class NLPTask(AI21Module):

    @classmethod
    def execute(cls, task_url, **params):
        client = AI21StudioClient(**params)
        return client.execute_http_request(method='POST', url=task_url, params=params)

    @classmethod
    def execute_sm_endpoint(cls, mandatory_fields=None, unsupported_fields=None, **params):
        endpoint_name = params.pop('sm_endpoint')
        call_name = f'Sagemaker {endpoint_name}'
        if mandatory_fields is not None:
            for mandatory_field in mandatory_fields:
                validate_mandatory_field(mandatory_field, call_name, params)
        if unsupported_fields is not None:
            for unsupported_field in unsupported_fields:
                validate_unsupported_field(key=unsupported_field, call_name=call_name, params=params)
        input_json = json.dumps(params)
        from ai21.SM_utils import invoke_sm_endpoint  # import here because boto3 exists only in SM mode
        return invoke_sm_endpoint(endpoint_name, input_json)



from typing import Optional, Dict
from ai21.errors import MissingApiKeyException, WrongInputTypeException
from ai21.http_client import HttpClient
from ai21.utils import get_global_configs, convert_to_ai21_object
from ai21.version import __version__


def build_ai21_user_agent():
    global_configs = get_global_configs()
    user_agent = f'ai21 studio SDK {__version__}'
    organization = global_configs['organization']
    if organization is not None:
        user_agent = f'{user_agent} organization: {organization}'
    application = global_configs['application']
    if application is not None:
        user_agent = f'{user_agent} application: {application}'
    return user_agent


def get_value(key: str, params: Dict, global_configs: Dict, expected_type: type):
    passed_value = params.get(key, None)
    if passed_value is not None:
        if not isinstance(passed_value, expected_type):
            raise WrongInputTypeException(key, expected_type, type(passed_value))
        return passed_value

    global_value = global_configs[key]
    if global_value is not None and not isinstance(global_value, expected_type):
        raise WrongInputTypeException(key, global_value, type(global_value))
    return global_value


class AI21StudioClient:
    def __init__(self, **params):
        global_configs = get_global_configs()
        api_key = get_value('api_key', params, global_configs, str)
        if api_key is None:
            raise MissingApiKeyException()
        self.api_key = api_key
        headers = self.build_default_headers()
        passed_headers = params.get('headers', None)
        if passed_headers is not None:
            if not isinstance(passed_headers, Dict):
                raise WrongInputTypeException('headers', Dict, type(passed_headers))
            headers.update(passed_headers)
        timeout_sec = get_value('timeout_sec', params, global_configs, int)
        num_retries = get_value('num_retries', params, global_configs, int)
        self.http_client = HttpClient(timeout_sec=timeout_sec, num_retries=num_retries, headers=headers)

    def build_default_headers(self):
        headers = {
            'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json',
            'User-Agent': build_ai21_user_agent()
        }
        return headers

    def execute_http_request(self, method: str, url: str, params: Optional[Dict] = None, files=None):
        response = self.http_client.execute_http_request(method=method, url=url, params=params, files=files)
        return convert_to_ai21_object(response)



import os

from setuptools import setup, find_packages

current_folder = os.path.abspath(os.path.dirname(__file__))
version_file_path = current_folder + '/ai21/version.py'

version_file = {}
with open(version_file_path, "rt") as f:
     exec(f.read(), version_file)
version = version_file["__version__"]

setup(
    name='ai21',
    version=version,
    license='MIT',
    author="ai21 labs",
    author_email='support@ai21.com',
    packages=find_packages(exclude=["tests", "tests.*"]),
    install_requires=[
        'requests',
    ],
    extras_require={'SM': 'boto3'},
)
